import React, { useState, useEffect, useRef, useContext } from 'react'
import { ConnectionService } from 'services/connectionService'
import { TriggerService } from 'services/triggersService'
import { AccountService } from 'services/accountService'

import { AppContext } from 'components/Contexts/AppContextProvider'

import { JobService, JobState } from 'services/jobsService'
import { FluidType } from 'enum/fluid.enum'

import { withClient, Client } from 'cozy-client'
import { translate } from 'cozy-ui/react/I18n'

import { getPicto, getAddPicto, getFuildType, getParamPicto } from 'utils/utils'
import chevronDown from 'assets/icons/ico/chevron-down.svg'
import chevronUp from 'assets/icons/ico/chevron-up.svg'

import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'

import failurePicto from '../KonnectorViewer/picto-failure.png'
import successPicto from '../KonnectorViewer/picto-success.png'
import pendingPicto from '../KonnectorViewer/picto-pending.png'
import KonnectorService from 'services/konnectorService'
import KonnectorStatusService from 'services/konnectorStatusService'
import IFluidConfig from 'services/IFluidConfig'

import KonnectorForm from 'components/ContentComponents/Konnector/KonnectorForm'
import KonnectorOAuthForm from 'components/ContentComponents/Konnector/KonnectorOAuthForm'
import KonnectorLoading from 'components/ContentComponents/Konnector/KonnectorLoading'
import KonnectorResult from 'components/ContentComponents/Konnector/KonnectorResult'

import { Account, Konnector, Trigger } from 'doctypes'

interface KonnectorViewerCardProps {
  fluidConfig: IFluidConfig
  konnector: Konnector
  trigger: any | null
  client: Client
  isParam: boolean
  t: Function
}

const KonnectorViewerCard: React.FC<KonnectorViewerCardProps> = ({
  fluidConfig,
  konnector,
  trigger,
  client,
  isParam,
  t,
}: KonnectorViewerCardProps) => {
  const [login, setLogin] = useState<string>('')
  const [password, setPassword] = useState<string>('')
  const [loading, setLoading] = useState<boolean>(false)
  const [updating, setUpdating] = useState<boolean>(false)
  const [konnectorAccountId, setAccountId] = useState<string>('')
  const [isKonnectorAcc, setAccount] = useState<boolean>(false)
  const [setActive, setActiveState] = useState('')
  const [setHeight, setHeightState] = useState('0px')
  const [updateTrigger, setUpdateTrigger] = useState<any>(null)
  const [updateDate, setUpdateDate] = useState<string>(
    new Date('0001-01-01T00:00:00Z').toDateString()
  )
  // const [frequency, setFrequency] = useState<string>('daily')
  const [error, setError] = useState<string>('')
  const [jobState, setJobState] = useState<string>('')
  const [launchedJob, setlaunchedJob] = useState<any>(null)
  const content: React.MutableRefObject<null> = useRef<null>(null)
  const type: string = fluidConfig.konnectorConfig.type
  const fluid: FluidType = getFuildType(type)
  const iconType = getPicto(fluid)
  const context = useContext(AppContext)

  const iconAddType = isParam ? getParamPicto(fluid) : getAddPicto(fluid)

  const toggleAccordion = () => {
    setActiveState(setActive === '' ? 'active' : '')
    setHeightState(
      setActive === 'active' &&
        content &&
        content.current &&
        content.current.scrollHeight
        ? '0px'
        : `${content.current.scrollHeight}px`
    )
  }

  const handleSubmit = async (e: React.FormEvent<HTMLFormElement>) => {
    try {
      e.preventDefault()
      setError('')
      setLoading(true)
      if (!login || !password) {
        setError('Please enter a login and password')
        setLoading(false)
        return null
      }
      setJobState(JobState.Running)
      const connectionService = new ConnectionService(
        client,
        fluidConfig.konnectorConfig.slug,
        login,
        password
      )
      const data = await connectionService.connectNewUser()
      setLoading(false)
      if (!data) {
        setError('Error during the user connection')
        setLoading(false)
        setJobState(JobState.Errored)
        return null
      }
      trigger = await KonnectorStatusService.getSingleTrigger(
        client,
        data.trigger_id
      )
      setUpdateTrigger(trigger)
      setlaunchedJob(data)
      setUpdateDate(new Date(data.queued_at).toDateString())
      setAccountId(data.message.account)
      setError('Connected')
      setAccount(true)
      return data
    } catch (error) {
      setLoading(false)
      setError(error.message)
    }
  }

  const deleteAccount = async () => {
    try {
      const account = await AccountService.getAccount(
        client,
        konnectorAccountId
      )
      if (!account) {
        setError('Error fetching account')
        setLoading(false)
      }
      const delAccount = await AccountService.deleteAccount(client, account)
      if (!delAccount) {
        setError('Error during deletion account')
        setLoading(false)
      }
      setError('')
      setJobState('')
      toggleAccordion()
      trigger = null
      setAccount(false)
      await context.refreshFluidTypes()
    } catch (error) {
      setLoading(false)
      setError(error.message)
    }
  }

  const updateKonnector = async () => {
    setError('')
    setJobState('')
    setUpdating(true)
    try {
      const account = await AccountService.getAccount(
        client,
        konnectorAccountId
      )
      const triggersServices = new TriggerService(client, account, konnector)
      if (updateTrigger !== null) {
        triggersServices.setTrigger(updateTrigger)
      } else {
        triggersServices.setTrigger(trigger)
      }
      const job = await triggersServices.launchTrigger()
      if (!job) {
        throw new Error(`Error during trigger launching`)
      }
      setlaunchedJob(job)
      setUpdateDate(new Date(job.queued_at).toDateString())
    } catch (error) {
      setUpdating(false)
      setError(error.message)
    }
  }

  const initOauthAccount = async (oauthAccountId: string) => {
    console.log('initOauthAccount', oauthAccountId)
    setError('')
    setJobState('')
    setLoading(true)
    try {
      const account = await AccountService.getAccount(client, oauthAccountId)
      console.log(konnector)
      const triggersServices = new TriggerService(client, account, konnector)
      const trigger: Trigger = await triggersServices.createTrigger()
      console.log(trigger)
      if (!trigger) {
        throw new Error(`Error during trigger creation`)
      }
      //Launch the creation trigger
      const job = await triggersServices.launchTrigger()
      if (!job) {
        throw new Error(`Error during trigger launching`)
      }
      console.log(job)
      setUpdateTrigger(trigger)
      setlaunchedJob(job)
      setUpdateDate(new Date(job.queued_at).toDateString())
      setAccountId(job.message.account)
      setError('Connected')
      setAccount(true)
      // await context.refreshFluidTypes()
    } catch (error) {
      setLoading(false)
      setError(error.message)
    }
  }

  const getIcon = (jobState: string) => {
    switch (jobState) {
      case JobState.Errored:
        return failurePicto
      // case JobState.Running:
      //   return pendingPicto
      // case JobState.Done:
      //   return successPicto
      default:
        return ''
    }
  }

  const getKonnectorStateMarkup = (jobState: string) => {
    const iconSrc = getIcon(jobState)
    if (iconSrc === '') return ''
    return <img className="state-icon" src={iconSrc}></img>
  }

  const jobStateCallBack = async (state: string) => {
    console.log('JobCallBack', state)
    setJobState(state)
    if (state !== JobState.Running) {
      setLoading(false)
      setUpdating(false)
      setJobState(state)
      context.refreshFluidTypes()
    }
  }

  useEffect(() => {
    let subscribed = true
    async function getTriggerData() {
      if (trigger || launchedJob) {
        const jobService = new JobService(client)
        let runningJob = null

        if (trigger) {
          let triggerState = null
          triggerState = TriggerService.fetchStateFromTrigger(trigger)
          const konnectorAcc = TriggerService.fetchKonnectorAccountFromTrigger(
            trigger
          )
          if (subscribed && konnectorAcc.konnector === konnector.slug) {
            setAccountId(konnectorAcc.account)
            setAccount(true)
          }
          if (subscribed && !launchedJob) setJobState(triggerState.status)
          if (triggerState.status === JobState.Running)
            runningJob = jobService.fetchJob(triggerState.last_executed_job_id)
        }
        if (launchedJob) runningJob = launchedJob
        if (runningJob) jobService.watch(runningJob, jobStateCallBack)
      }
    }
    getTriggerData()
    return () => {
      subscribed = false
    }
  }, [trigger, launchedJob])

  return (
    <>
      <div className={`accordion ${setActive}`}>
        <div>
          <div
            className={`accordion-header ${setActive}`}
            onClick={toggleAccordion}
          >
            <div className="accordion-icon">
              {!isKonnectorAcc ? (
                <StyledIcon className="icon" icon={iconAddType} size={49} />
              ) : (
                <StyledIcon className="icon" icon={iconType} size={49} />
              )}
            </div>
            <div className="state-picto">
              {getKonnectorStateMarkup(jobState)}
            </div>
            <div className="accordion-info">
              <div className="accordion-title text-16-normal">
                {!isKonnectorAcc
                  ? t('KONNECTORCONFIG.LABEL_CONNECTTO_' + FluidType[fluid])
                  : t('FLUID.' + FluidType[fluid] + '.LABEL')}
              </div>
            </div>
            <StyledIconButton icon={setActive ? chevronUp : chevronDown} />
          </div>
          <div
            ref={content}
            style={{ maxHeight: `${setHeight}` }}
            className={`accordion-content ${setActive}`}
          >
            {!isKonnectorAcc ? (
              !fluidConfig.konnectorConfig.oauth ? (
                <KonnectorForm
                  fluidConfig={fluidConfig}
                  login={login}
                  setLogin={setLogin}
                  password={password}
                  setPassword={setPassword}
                  loading={loading}
                  error={error}
                  handleSubmit={handleSubmit}
                />
              ) : (
                <KonnectorOAuthForm
                  konnector={konnector}
                  onSuccess={initOauthAccount}
                  loading={loading}
                />
              )
            ) : loading ? (
              <KonnectorLoading />
            ) : (
              <KonnectorResult
                date={
                  trigger
                    ? TriggerService.fetchDateFromTrigger(trigger)
                    : updateDate
                }
                updating={updating}
                errored={jobState === JobState.Errored}
                updateKonnector={updateKonnector}
                deleteAccount={deleteAccount}
              />
            )}
          </div>
        </div>
      </div>
    </>
  )
}
export default translate()(withClient(KonnectorViewerCard))
